@@ -1,8 +1,7 @@
 from EEETools.MainModules import ArrayHandler
 
 
-def update_exergy_values(array_handler: ArrayHandler, exergy_list:list) -> ArrayHandler:
-
+def update_exergy_values(array_handler: ArrayHandler, exergy_list: list) -> ArrayHandler:
     """
     :param array_handler: an array_handler
     :param exergy_list: a list of dictionaries having the following keys:
@@ -12,7 +11,6 @@ def update_exergy_values(array_handler: ArrayHandler, exergy_list:list) -> Array
 
     """
     for stream in exergy_list:
-
         connection = array_handler.find_connection_by_index(stream["index"])
         connection.exergy_value = stream["value"]
 
@@ -21,17 +19,45 @@ def update_exergy_values(array_handler: ArrayHandler, exergy_list:list) -> Array
 
 def get_result_data_frames(array_handler: ArrayHandler) -> dict:
 
-    # Stream Solution Data frame generation
-    stream_data = {"Stream": list(),
-                   "Name": list(),
-                   "Exergy Value [kW]": list(),
-                   "Specific Cost [€/kJ]": list(),
-                   "Specific Cost [€/kWh]": list(),
-                   "Total Cost [€/s]": list()}
+    if array_handler.options.calculate_component_decomposition:
+
+        return {
+
+            "Stream Out": __get_stream_data_frame(array_handler),
+            "Comp Out": __get_comp_data_frame(array_handler),
+            "Cost Dec Out": __get_cost_dec_data_frame(array_handler),
+            "Eff Out": __get_useful_data_frame(array_handler)
+
+        }
+
+    else:
+
+        return {
+
+            "Stream Out": __get_stream_data_frame(array_handler),
+            "Comp Out": __get_comp_data_frame(array_handler),
+            "Eff Out": __get_useful_data_frame(array_handler)
+
+        }
+
+
+def __get_stream_data_frame(array_handler: ArrayHandler):
+
+    stream_data = {
+
+        "Stream": list(),
+        "Name": list(),
+        "Exergy Value [kW]": list(),
+        "Specific Cost [€/kJ]": list(),
+        "Specific Cost [€/kWh]": list(),
+        "Total Cost [€/s]": list()
+
+    }
 
     for conn in array_handler.connection_list:
 
         if not conn.is_internal_stream:
+
             stream_data["Stream"].append(conn.index)
             stream_data["Name"].append(conn.name)
             stream_data["Exergy Value [kW]"].append(conn.exergy_value)
@@ -39,27 +65,35 @@ def get_result_data_frames(array_handler: ArrayHandler) -> dict:
             stream_data["Specific Cost [€/kWh]"].append(conn.rel_cost * 3600)
             stream_data["Total Cost [€/s]"].append(conn.rel_cost * conn.exergy_value)
 
-    # Components Data frame generation
-    comp_data = {"Name": list(),
-                 "Comp Cost [€/s]": list(),
+    return stream_data
 
-                 "Exergy_fuel [kW]": list(),
-                 "Exergy_product [kW]": list(),
-                 "Exergy_destruction [kW]": list(),
-                 "Exergy_loss [kW]": list(),
-                 "Exergy_dl [kW]": list(),
 
-                 "Fuel Cost [€/kWh]": list(),
-                 "Fuel Cost [€/s]": list(),
-                 "Product Cost [€/kWh]": list(),
-                 "Product Cost [€/s]": list(),
-                 "Destruction Cost [€/kWh]": list(),
-                 "Destruction Cost [€/s]": list(),
+def __get_comp_data_frame(array_handler: ArrayHandler):
 
-                 "eta": list(),
-                 "r": list(),
-                 "f": list(),
-                 "y": list()}
+    comp_data = {
+
+        "Name": list(),
+        "Comp Cost [€/s]": list(),
+
+        "Exergy_fuel [kW]": list(),
+        "Exergy_product [kW]": list(),
+        "Exergy_destruction [kW]": list(),
+        "Exergy_loss [kW]": list(),
+        "Exergy_dl [kW]": list(),
+
+        "Fuel Cost [€/kWh]": list(),
+        "Fuel Cost [€/s]": list(),
+        "Product Cost [€/kWh]": list(),
+        "Product Cost [€/s]": list(),
+        "Destruction Cost [€/kWh]": list(),
+        "Destruction Cost [€/s]": list(),
+
+        "eta": list(),
+        "r": list(),
+        "f": list(),
+        "y": list()
+
+    }
 
     for block in array_handler.block_list:
 
@@ -83,7 +117,8 @@ def get_result_data_frames(array_handler: ArrayHandler) -> dict:
                 comp_data["Product Cost [€/s]"].append(block.output_cost * block.exergy_analysis["product"])
                 comp_data["Destruction Cost [€/s]"].append(
 
-                    block.coefficients["c_dest"] * (block.exergy_analysis["distruction"] + block.exergy_analysis["losses"])
+                    block.coefficients["c_dest"] * (
+                            block.exergy_analysis["distruction"] + block.exergy_analysis["losses"])
 
                 )
 
@@ -107,16 +142,43 @@ def get_result_data_frames(array_handler: ArrayHandler) -> dict:
                 comp_data["f"].append(0)
                 comp_data["y"].append(0)
 
-    # Output Stream Data frame generation
-    useful_data = {"Stream": list(),
-                   "Name": list(),
-                   "Exergy Value [kW]": list(),
-                   "Specific Cost [€/kJ]": list(),
-                   "Specific Cost [€/kWh]": list(),
-                   "Total Cost [€/s]": list()}
+    return comp_data
+
+
+def __get_cost_dec_data_frame(array_handler: ArrayHandler):
+
+    cost_dec_data = {"Name": list()}
+
+    for block in array_handler.block_list:
+
+        if not block.is_support_block:
+
+            cost_dec_data["Name"].append(block.name)
+
+            for name in block.output_cost_decomposition.keys():
+
+                if name not in cost_dec_data.keys():
+                    cost_dec_data.update({name: list()})
+
+                cost_dec_data[name].append(block.output_cost_decomposition[name])
+
+    return cost_dec_data
+
+
+def __get_useful_data_frame(array_handler: ArrayHandler):
+
+    useful_data = {
+
+        "Stream": list(),
+        "Name": list(),
+        "Exergy Value [kW]": list(),
+        "Specific Cost [€/kJ]": list(),
+        "Specific Cost [€/kWh]": list(),
+        "Total Cost [€/s]": list()
+
+    }
 
     for conn in array_handler.useful_effect_connections:
-
         useful_data["Stream"].append(conn.index)
         useful_data["Name"].append(conn.name)
         useful_data["Exergy Value [kW]"].append(conn.exergy_value)
@@ -124,6 +186,4 @@ def get_result_data_frames(array_handler: ArrayHandler) -> dict:
         useful_data["Specific Cost [€/kWh]"].append(conn.rel_cost * 3600)
         useful_data["Total Cost [€/s]"].append(conn.rel_cost * conn.exergy_value)
 
-    return {"Stream Out": stream_data,
-            "Comp Out": comp_data,
-            "Eff Out": useful_data}
\ No newline at end of file
+    return useful_data
